#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <ctype.h>
#include <unistd.h>

#include "ftdi.h"

static struct ftdi_context handle;


static int
Fail1(int status, const char *file, int line, const char *sline)
{
  const char *er = "Unknown error";

  if (status >= 0)
    return status;

  er = ftdi_get_error_string (&handle);

  printf("%s:%d: Error %s (%d) doing %s\n", file, line, er, status, sline);
  exit(1);
}

#define Fail(x) Fail1(x,__FILE__,__LINE__,#x)

static void
serial_close ()
{
  ftdi_usb_reset (&handle);
  ftdi_usb_close (&handle);
  ftdi_deinit (&handle);
}

int tic = 0;

int ft_getc ()
{
  int rv;
  unsigned char buf[1];
  while (1)
    {
      rv = ftdi_read_data (&handle, buf, 1);
      if (rv != 1)
	usleep (10);
      else
	{
	  if ((buf[0] & 0xfc) == 0xcc)
	    {
	      do {
		tic ++;
	      } while (tic & 7 != buf[0] & 7);
	      return -1;
	    }

	  if (buf[0] == 0xc8)
	    {
	      do {
		rv = ftdi_read_data (&handle, buf, 1);
	      } while (rv != 1);
	      return buf[0] | 0x80;
	    }
	  else
	    return buf[0];
	}
    }
}

unsigned char prev_buf[100];
unsigned char new_buf[100];
int nb = 0, pb = 0;

main(int argc, char **argv)
{
  int status, rv;
  ftdi_init (&handle);
  int do_all = 0;

  atexit (serial_close);

  status = ftdi_usb_open_desc (&handle, 0x0403, 0x6001, "TTL232R", NULL);


  if (argc > 1 && strcmp (argv[1], "-a") == 0)
    do_all = 1;

  if (argc > 1 && strcmp (argv[1], "-r") == 0)
    {
      Fail (ftdi_set_baudrate (&handle, 115200));
      printf("raw capture 115200 8N1, ctrl-c to exit\n");
      while (1)
	{
	  unsigned char buf[100];
	  while (1)
	    {
	      rv = ftdi_read_data (&handle, buf, 100);
	      if (rv < 1)
		usleep (10);
	      else
		write (1, buf, rv);
	    }
	}
    }

  Fail (ftdi_set_baudrate (&handle, 1263157));
  while (1)
    {
      int b = ft_getc ();

      if (b == -1)
	{
	  if (nb != pb || memcmp (new_buf, prev_buf, nb) || do_all)
	    {
	      int i, off, rpm;
#if 0
	      off = new_buf[1] * 256 + new_buf[0];
	      off = (int16_t) off;
	      rpm = new_buf[3] * 256 + new_buf[2];
	      rpm = (int16_t) rpm;
	      printf("%6d %6d ", off, rpm);
#endif
	      printf("%.4g ", tic / 20000.0);
	      for (i=0; i<nb; i++)
		{
		  printf("%02x", new_buf[i]);
#if 0
		  if (isgraph (new_buf[i]))
		    printf("\033[34m%c\033[0m", new_buf[i]);
		  else
		    printf(" ");
#endif
		  printf(" ");
		}
	      printf("\n");
	    }
	  memcpy (prev_buf, new_buf, nb);
	  pb = nb;
	  nb = 0;
	}
      else if (nb < 99)
	new_buf[nb++] = b;
    }
}
