#include <stdio.h>
#include <math.h>
#include "iodefine.h"
#include "bldc.h"

static double cosine_table[] = {
#include "cosines.h"
};

/* These numbers give us a cycle time of exactly 20 KHz (50 uS) */
#define DEAD 9
#define CYCLE 3990
#define MAXPOWER (CYCLE/2)

static int prev_adc_channel = 0;
static int next_adc_channel = 0;

volatile float motor_current = 0;
volatile float torque_current = 0;

volatile int cpwm_torque_applied = 0;

int uzero, vzero, wzero;
int usign, vsign, wsign;
float u_curr, v_curr, w_curr;
float cos_angle, sin_angle;


double
dcos (int pos)
{
  pos *= 2;
  pos %= COS_TABLE_SIZE*4;
  if (pos < 0)
    pos += COS_TABLE_SIZE*4;
  switch (pos / COS_TABLE_SIZE) {
  case 0:
    return cosine_table[pos];
  case 1:
    return - cosine_table[COS_TABLE_SIZE*2 - pos];
  case 2:
    return - cosine_table[pos - COS_TABLE_SIZE*2];
  case 3:
    return cosine_table[COS_TABLE_SIZE*4 - pos];
  }
}

#if 0
/* Used to "scope" out the motor currents on tty2.  */
static void __attribute__((interrupt))
s12ad_handler ()
{
  tty2_putc (0xff);
  tty2_putc (UCURR >> 8);
  tty2_putc (VCURR >> 8);
  tty2_putc (WCURR >> 8);
  tty2_putc (next_adc_channel);
}
#endif


volatile int8_t encoder_direction = 0;
static short old_encoder_count = 0;
volatile  int raw_current;

static void __attribute__((interrupt))
cpwm_peak_handler ()
{
  int pos = encoder_ang (); /* 0..1999 */
  short new_encoder = MTU1.TCNT;
  float sum;

  MTU3.TSR.BIT.TGFA = 0;

  if (new_encoder != old_encoder_count)
    {
      int new_direction = encoder_direction;
      if (new_encoder > old_encoder_count)
	new_direction = 1;
      else if (new_encoder < old_encoder_count)
	new_direction = -1;
      if (new_direction != encoder_direction && encoder_direction)
	encoder_direction = 0;
      else
	encoder_direction = new_direction;
      old_encoder_count = new_encoder;
    }

#define adc_to_curr(adc, zero) ((int)(unsigned short)adc - zero) / -2228.224

  cos_angle = dcos (pos);
  sin_angle = dcos (pos - 500);

  u_curr = adc_to_curr (UCURR, uzero);
  v_curr = adc_to_curr (VCURR, vzero);
  w_curr = adc_to_curr (WCURR, wzero);
  sum = (u_curr + v_curr + w_curr) / 3;
  u_curr -= sum;
  v_curr -= sum;
  w_curr -= sum;
  motor_current = (fabs(u_curr) + fabs(v_curr) + fabs(w_curr))/2;

  {
    static int counts = 0;
    if (TOO_HOT)
      {
	counts ++;
	if (counts > 200)
	  {
	    motor_estop ();
	    counts = 0;
	  }
      }
    else
      counts = 0;
  }

  motor_pwm_update();

  limit_pwm_update ();
}

static void __attribute__((interrupt))
cpwm_gap_handler ()
{
  static int which_adc = 0;
  which_adc = (which_adc + 1) % 3;
  //  S12AD1.ADANS.BIT.CH = which_adc;
  S12AD1.ADCSR.BIT.ADST = 1; /* start current measurement */
  MTU4.TSR.BIT.TCFV = 0;
}

void
cpwm_init()
{
  
  /* Figure 17.38 "Example of Complementary PWM Mode (Unit 0) Setting
     Procedure" */

  /* [1] Clear bits CST3 and CST4 in the timer start register (TSTRA)
         to 0 to stop timer counter (TCNT) operation. Specify
         complementary PWM mode while MTU3.TCNT and MTU4.TCNT are
         stopped. */

  MTU.TSTRA.BIT.CST3 = 0;
  MTU.TSTRA.BIT.CST4 = 0;

  /* [2] Set the same counter clock and clock edge for channels 3 and
         4 with bits TPSC[2:0] and bits CKEG[1:0] in the timer control
         register (TCR). Use bits CCLR[2:0] to set synchronous
         clearing only when restarting by synchronous clearing with
         another channel during complementary PWM mode operation. */

  MTU3.TCR.BIT.TPSC = 0;	/* ICLK/1 */
  MTU3.TCR.BIT.CKEG = 0;
  MTU3.TCR.BIT.CCLR = 3;	/* Sync */

  MTU4.TCR.BIT.TPSC = 0;	/* PCLK/1 */
  MTU4.TCR.BIT.CKEG = 0;
  MTU4.TCR.BIT.CCLR = 3;	/* Sync */

  /* [3] When performing brushless DC motor control, set bit BDC in
         the timer gate control register (TGCRA) and set the feedback
         signal input source and output chopping or gate signal direct
         output. */

  MTU.TGCRA.BIT.BDC = 0;
  MTU.TGCRA.BIT.P = 1;
  MTU.TGCRA.BIT.N = 1;
  MTU.TGCRA.BIT.FB = 0;

  /* [4] Set the dead time in MTU3.TCNT. Set MTU4.TCNT to 0000h. */

  MTU3.TCNT = DEAD;
  MTU4.TCNT = 0;

  /* [5] Set only when restarting by synchronous clearing with another
         channel during complementary PWM mode operation. In this
         case, synchronize the channel generating the synchronous
         clear with channels 3 and 4 using the timer synchronous
         register (TSYRA). */

  MTU.TSYRA.BIT.SYNC3 = 0;
  MTU.TSYRA.BIT.SYNC4 = 0;

  /* [6] Set the output PWM duty cycles in the duty registers
         (MTU3.TGRB, MTU4.TGRA, and MTU4.TGRB) and buffer registers
         (MTU3.  TGRD, MTU4.TGRC, and MTU4.TGRD). Set the same value
         in the buffer registers and the duty registers. */

  MTU3.TGRB = 1;	/* Phase 1 */
  MTU3.TGRD = 1;	/* Phase 1 buffer */
  MTU4.TGRA = 1;	/* Phase 2 */
  MTU4.TGRC = 1;	/* Phase 2 buffer */
  MTU4.TGRB = 1;	/* Phase 3 */
  MTU4.TGRD = 1;	/* Phase 3 buffer */

  /* [7] This setting is necessary only when no dead time should be
         generated. Make appropriate settings in the timer dead time
         enable register (TDER) so that no dead time is generated. */

  MTU.TDERA.BYTE = 0;

  /* [8] Set the dead time in the dead time register (TDDRA), 1/2 the
         carrier cycle in the carrier cycle data register (TCDRA) and
         carrier cycle buffer register (TCBRA), and 1/2 the carrier
         cycle plus the dead time in MTU3.TGRA and MTU3.TGRC. When no
         dead time generation is selected, set 1 in TDDRA and 1/2 the
         carrier cycle + 1 in MTU3.TGRA and MTU3.TGRC. */

  MTU.TDDRA = DEAD;
  MTU.TCDRA = CYCLE/2;
  MTU.TCBRA = CYCLE/2;
  MTU3.TGRA = CYCLE/2 + DEAD;
  MTU3.TGRC = CYCLE/2 + DEAD;

  /* [9] Enable or disable toggle output synchronized with the PWM
         cycle using bit PSYE in the timer output control register 1
         (TOCR1A), and set the PWM output level with bits OLSP and
         OLSN. When specifying the PWM output level by using TOLBR as
         a buffer for MTU2.TOCR, see figure 17.3. */

  MTU.TOCR1A.BIT.PSYE = 1; /* this conflicts with ENCA */
  MTU.TOCR1A.BIT.OLSP = 1;
  MTU.TOCR1A.BIT.OLSN = 0;
  MTU.TOCR1A.BIT.TOCS = 1;
  MTU.TOCR1A.BIT.TOCL = 0;

  MTU.TOCR2A.BYTE = 0x3f;

  /* [10] Select complementary PWM mode in the timer mode register
          (MTU3.TMDR). Set the BFA bit in MTU3.TMDR to 1 to use TGRA
          and TGRC together for buffer operation, and the BFB bit in
          MTU3.TMDR to 1 to use TGRB and TGRD together for buffer
          operation, if necessary. Set the initial Value in
          MTU4.TMDR.  */

  MTU3.TMDR.BYTE = 0x3f; /* transfer at both, buffered */
  MTU4.TMDR.BYTE = 0x0f; /* transfer at both */

  /* [11] Set enabling/disabling of PWM waveform output pin output in
          the timer output master enable register (TOERA). */

  MTU.TOERA.BYTE = 0x00;

  /* [12] Set the port control register and the port I/O register. */

  PORT7.DR.BYTE = 0;
  PORT7.DDR.BYTE = UH | UL | VH | VL | WH | WL;

  /* [13] Set bits CST3 and CST4 in TSTRA to 1 simultaneously to start
          the count operation. */

  MTU.TSTRA.BYTE |= 0xc0;

#if 1
  /* pwm cycle peak interrupt */
  _vectors[VECT(MTU3,TGIA3)] = cpwm_peak_handler;
  IEN(MTU3,TGIA3) = 1;
  IPR(MTU3,TGIA3) = 9;
  MTU3.TIER.BIT.TGIEA = 1;
#endif

#if 1
  /* pwm cycle gap interrupt */
  _vectors[VECT(MTU4,TCIV4)] = cpwm_gap_handler;
  IEN(MTU4,TCIV4) = 1;
  IPR(MTU4,TCIV4) = 9;
  MTU4.TIER.BIT.TCIEV = 1;
#endif

  {
    int u, v, w;
    int ou, ov, ow;
    long otime, timeout;
    /* zero out the current sensors */
    next_adc_channel = UL;
    MTU.TOERA.BYTE = 0x00;
    PORT7.DR.BYTE = UL | VL | WL;
    S12AD1.ADANS.WORD = 0x2000;
    S12AD1.ADCSR.BYTE = 0x4c;
    S12AD1.ADCSR.BYTE = 0xcc;
    S12AD1.ADCER.BIT.ADRFMT = 1;
    ou = 0;
    ov = 0;
    ow = 0;
    otime = msec;
    timeout = msec + 1000;

    while (1)
      {
	/* variation of up to 0x200 = within 0.11 amps of zero */
#define D 0x200
#define R(i,oi,IC) \
	i = (int)(unsigned short)IC;		\
	if (abs (i - oi) > D) { \
	  oi = i; otime = msec; }		\
	else oi = oi * 7/8 + i * 1/8; 
	R(u, ou, UCURR);
	R(v, ov, VCURR);
	R(w, ow, WCURR);
	if (msec - otime > 100)
	  break;
	if (msec - timeout > 0)
	  break;
      }
    uzero = ou;
    vzero = ov;
    wzero = ow;
    printf("zero: %04x %04x %04x\n", uzero, vzero, wzero);
    printf("raw %d motor %f\n", raw_current, motor_current);

    if (uzero < 0x7000 || uzero > 0x9000
	|| vzero < 0x7000 || vzero > 0x9000
	|| wzero < 0x7000 || wzero > 0x9000)
      {
	printf("Invalid zero current, halt.\n");
	led_flash (led_red);
	while (1)
	  {
	    printf("  U %04x V %04x W %04x   \r", UCURR, VCURR, WCURR);
	    fflush (stdout);
	    wait_msec (100);
	  }
      }
  }

#if 1
  /* Current sense ADC */
  S12AD1.ADANS.WORD = 0x2000; /* no gain, gain bypassed, WCURR selected */
  S12AD1.ADCER.BIT.ADRFMT = 1;
  S12AD1.ADCSR.BYTE = 0x2c; /* single scan mode, PCLK, no trigger, internal trigger */

#else
  /* alternate adc - scan all three and interupt, output to tty2 */
  S12AD1.ADANS.WORD = 0x2000; /* no gain, gain bypassed, WCURR selected */
  S12AD1.ADCER.BIT.ADRFMT = 1;
  S12AD1.ADCSR.BYTE = 0x3c; /* scan mode, PCLK, no trigger, internal trigger, interrupt */
  _vectors[VECT(S12AD1,S12ADI1)] = s12ad_handler;
  IEN(S12AD1,S12ADI1) = 1;
  IPR(S12AD1,S12ADI1) = 9;
#endif
}

static int
power2level (int x)
{
  int rv;
  /* Turn -32768..32767 into MAXPOWER/2..0 */
  rv = MAXPOWER/2 - x * (MAXPOWER/2) / 32768;
  rv = MAXPOWER/2 - x * (MAXPOWER/2) / 32768;
  if (rv < 0)
    rv = 0;
  if (rv >= MAXPOWER)
    rv = CYCLE/2+DEAD;
  return rv;
}

/* U V W are -32768..32767 */
void
cpwm_levels (int u, int v, int w)
{
  unsigned int pu, pv, pw, pmax, motor_current_i;
  int adj;

  if (SD_ESTOP_IN == 0)
    {
      MTU3.TGRD = 0;
      MTU4.TGRC = 0;
      MTU4.TGRD = 0;
    }

  pu = power2level (u);
  pv = power2level (v);
  pw = power2level (w);

#if 1
  if (pu > pv && pu > pw)
    {
      pmax = pu;
      next_adc_channel = UL;
    }
  else if (pv > pw)
    {
      pmax = pv;
      next_adc_channel = VL;
    }
  else
    {
      pmax = pw;
      next_adc_channel = WL;
    }
  adj = (CYCLE/2+DEAD) - pmax;
  pu += adj;
  pv += adj;
  pw += adj;
  if (pu >= CYCLE/2) pu = CYCLE/2+DEAD;
  if (pv >= CYCLE/2) pv = CYCLE/2+DEAD;
  if (pw >= CYCLE/2) pw = CYCLE/2+DEAD;
#endif

  MTU3.TGRD = pu;
  MTU4.TGRC = pv;
  MTU4.TGRD = pw;
}

static int old_pos = 8000;

int cpwm_step;
static int current_limit_timeout = 0;

/* This is the dumb torque routine.  "torque" is a -max..max value and
   we PWM according to that and the angle, nothing else.  */
void
cpwm_torque (int torque)
{
  int pos = encoder_ang (); /* 0..1999 */
  float ucos, vcos, wcos;

  if (torque > MAX_TORQUE) torque = MAX_TORQUE;
  if (torque < -MAX_TORQUE) torque = -MAX_TORQUE;

  /* pos is 0 when the rotor is aligned with the +U coil.  */

  ucos = -torque * dcos(pos - ULEAD);
  vcos = -torque * dcos(pos - VLEAD);
  wcos = -torque * dcos(pos - WLEAD);

  /* This is counter-intuitive, but we're measuring positive current
     on the *grounded* motor phases, which have the lower levels.  */
  usign = (ucos > 0) ? -1 : 1;
  vsign = (vcos > 0) ? -1 : 1;
  wsign = (wcos > 0) ? -1 : 1;

#if 1
  if (motor_current > 3)
    {
      if (current_limit_timeout > 20) /* 1 mS */
	torque = 0;
      else
	current_limit_timeout ++;
    }
  else if (motor_current < 1)
    current_limit_timeout = 0;
#endif

  cpwm_torque_applied = torque;

  if (torque == 0)
    {
      MTU3.TGRD = 
	MTU4.TGRC = 
	MTU4.TGRD = CYCLE/2+DEAD;
      return;
    }

  /* A positive torque needs to cause a positive change in the
     encoders.  */
  cpwm_levels (ucos, vcos, wcos);

#if 1
  if (tty2_idle ())
    {
      tty2_put_marker ();
      tty2_putw_encoded (pos);
      tty2_putw_encoded (torque);
      tty2_putw_encoded (ucos);
      tty2_putw_encoded (vcos);
      tty2_putw_encoded (wcos);
    }
#endif
}

void
bldc_run ()
{
  MTU.TOERA.BYTE = 0x3f;
}

void
bldc_stop ()
{
  cpwm_torque_applied = 0;
  MTU.TOERA.BYTE = 0x00;
  PORT7.DR.BYTE = 0;
  while (encoder_rpm () > 500)
    ;
  PORT7.DR.BYTE = UL | VL | WL;
}
