#include <stdio.h>
#include <math.h>
#include "bldc.h"
#include "motor.h"

int limit1_triggered = 0;
int limit2_triggered = 0;
int limit3_triggered = 0;
int limit4_triggered = 0;
Position limit1_position;
Position limit2_position;
Position limit3_position;
Position limit4_position;

static volatile enum {
  No_Event,
  EV_1L, EV_1H,
  EV_2L, EV_2H,
  EV_3L, EV_3H,
  EV_4L, EV_4H,
} event;

static const char * const messages[] = {
  "No Event",
  "1L", "1H",
  "2L", "2H",
  "3L", "3H",
  "4L", "4H",
};

static void __attribute__((interrupt))
limit1_handler ()
{
  if (PORT9.PORT.BIT.B1)
    event = EV_1H;
  else
    event = EV_1L;
  limit1_triggered = 1;
  limit1_position = motor_position;
}

static void __attribute__((interrupt))
limit2_handler ()
{
  if (PORT9.PORT.BIT.B2)
    event = EV_2H;
  else
    event = EV_2L;
  limit2_triggered = 1;
  limit2_position = motor_position;
}

static void __attribute__((interrupt))
limit3_handler ()
{
  if (PORT9.PORT.BIT.B3)
    event = EV_3H;
  else
    event = EV_3L;
  limit3_triggered = 1;
  limit3_position = motor_position;
}

static void __attribute__((interrupt))
limit4_handler ()
{
  if (PORT9.PORT.BIT.B4)
    event = EV_4H;
  else
    event = EV_4L;
  limit4_triggered = 1;
  limit4_position = motor_position;
}

limit_reset ()
{
  MTU7.TSR.BIT.TGFA = 0;
  MTU7.TSR.BIT.TGFB = 0;
  MTU7.TSR.BIT.TGFC = 0;
  MTU6.TSR.BIT.TGFD = 0;
}

main()
{
  int i = 0;
  int j = 0;
  bldc_init ();

  tty_puts("\033[1;1H\033[2J\n\nHello from Limit 1!\n");

  /* Timers are free-running, and interrupt on input capture.  The
     input capture is on both edges, so any input change generates an
     interrupt.  */

  MTU6.TCR.BYTE = 0x05; /* count on ICLK/1024, one edge, no clearing */
  MTU6.TCR.BIT.CCLR = 0; /* no counter clearing */
  MTU6.TMDR.BYTE = 0; /* normal operation */

  MTU7.TCR.BYTE = 0x05; /* count on ICLK/1024, one edge, no clearing */
  MTU7.TCR.BIT.CCLR = 0; /* no counter clearing */
  MTU7.TMDR.BYTE = 0; /* normal operation */

  /* LIMIT 1 is MTIOC7C port P9.1 */

  MTU7.TIORL.BIT.IOC = 10; /* input capture at both edges */
  PORT9.ICR.BIT.B1 = 1; /* the port is an input */
  MTU7.TIER.BIT.TGIEC = 1; /* enable interrupt */
  _vectors[VECT(MTU7,TGIC7)] = limit1_handler;
  IEN(MTU7,TGIC7) = 1; /* enable interrupt */
  IPR(MTU7,TGIC7) = 8; /* priority */

  /* LIMIT 2 is MTIOC6D port P9.2 */

  MTU6.TIORL.BIT.IOD = 10; /* input capture at both edges */
  PORT9.ICR.BIT.B2 = 1; /* the port is an input */
  MTU6.TIER.BIT.TGIED = 1; /* enable interrupt */
  _vectors[VECT(MTU6,TGID6)] = limit2_handler;
  IEN(MTU6,TGID6) = 1; /* enable interrupt */
  IPR(MTU6,TGID6) = 8; /* priority */

  /* LIMIT 3 is MTIOC7B port P9.3 */

  MTU7.TIORH.BIT.IOB = 10; /* input capture at both edges */
  PORT9.ICR.BIT.B3 = 1; /* the port is an input */
  MTU7.TIER.BIT.TGIEB = 1; /* enable interrupt */
  _vectors[VECT(MTU7,TGIB7)] = limit3_handler;
  IEN(MTU7,TGIB7) = 1; /* enable interrupt */
  IPR(MTU7,TGIB7) = 8; /* priority */

  /* LIMIT 4 is MTIOC7A port P9.4 */

  MTU7.TIORH.BIT.IOA = 10; /* input capture at both edges */
  PORT9.ICR.BIT.B4 = 1; /* the port is an input */
  MTU7.TIER.BIT.TGIEA = 1; /* enable interrupt */
  _vectors[VECT(MTU7,TGIA7)] = limit4_handler;
  IEN(MTU7,TGIA7) = 1; /* enable interrupt */
  IPR(MTU7,TGIA7) = 8; /* priority */

  MTU.TSTRB.BIT.CST7 = 1;
  MTU.TSTRB.BIT.CST6 = 1;

  while (1)
    {
      if (event != No_Event)
	{
	  int e = event;
	  event = No_Event;
	  limit_reset ();
	  cprintf("%w %w %w ", i, MTU7.TCNT, MTU7.TGRA);
	  i++;
	  tty_puts (messages[e]);
	  tty_putc('\n');
	}
      cprintf("\r %w %w %w %w ", MTU7.TCNT, MTU7.TSR.BYTE, MTU7.TIER.BYTE, ICU.IER[IER_MTU7_TGIA7]);
      j++;
      BLUE =  (j & 0x040) ? 1 : 0;
      RED =   (j & 0x080) ? 1 : 0;
      GREEN = (j & 0x100) ? 1 : 0;
    }

  exit (0);
}
