#include "iodefine.h"
#include "osboard.h"

unsigned char led_val;

void
set_led (unsigned char newval)
{
  LED = led_val = newval;
}

unsigned char
get_led (unsigned char newval)
{
  return led_val;
}

void
osboard_init ()
{
  MSTP(S12AD) = 0;
  MSTP(DA) = 0;
  MSTP(MTU0) = 0;
  MSTP(MTU1) = 0;
  MSTP(SCI0) = 0;
  MSTP(SCI1) = 0;
  MSTP(SCI6) = 0;
  MSTP(USB0) = 0;
  MSTP(EDMAC) = 0;
  MSTP(RAM0) = 0;
  MSTP(RAM1) = 0;

  /* We start with a 12 MHz crystal.  */
  /* 0 = x8  1 = x4  2 = x2  3 = x1 */
  /* ICLK: x8 =  96 MHz  */
  /* PCLK: x4 =  48 MHz   */
  /* BCLK: x4 =  48 MHz   */
  /*                       I B P	*/
  SYSTEM.SCKCR.LONG = 0x00010100;
  /*	0 = 96 MHz
	1 = 48 MHz
	2 = 24 MHz
	3 = 12 MHz
  */

  /* Make sure 32768 Hz clock is running.  */
  SYSTEM.SUBOSCCR.BIT.SUBSTOP = 0;


  PORT0.DDR.BYTE = SD_DO_MASK | SD_CLK_MASK | SD_POWER_MASK;	/* P0.3, P0.5 = DAC, others SD */
  PORT2.DDR.BYTE = 0xC0;	/* P2.6 = nCS5 P2.7 = CTS0 */
  PORT3.DDR.BYTE = 0x08;	/* P3.3 = RTS0 */
  PORT4.DDR.BYTE = SD_CS_MASK;	/* P4.2 = IR, P4.3 = THERM, P4.4 = AMBIENT */
  PORT5.DDR.BYTE = 0x09;	/* P5.0 = nWR0, P5.4 = 1wire, P5.3 = BCLK  */
  PORT6.DDR.BYTE = 0x22;	/* P6.1 = nSdcs, P6.5 = CKE */
  PORT7.DDR.BYTE = 0x19;	/* P7.0 = CLK, P7.3 = CS3, P7.4 = CS4 */
  PORT9.DDR.BYTE = 0x02;	/* P9.1 = PROGB */
  PORTD.DDR.BYTE = 0xff;	/* LEDs */

  PORT1.DR.BYTE = 0xaa;
  PORT2.DR.BYTE = 0x00;
  PORT5.DR.BYTE = 0x00;
  PORT6.DR.BYTE = 0xff;
  PORT9.DR.BYTE = 0x00;

  PORT7.DR.BYTE = 0xff;
  PORT7.DR.BYTE = 0x00;
  PORT7.DR.BYTE = 0xff;
  PORT7.DR.BYTE = 0x00;

  PORT0.ICR.BYTE = SD_DI_MASK;
  PORT4.ICR.BYTE = SD_PRESENT_MASK;
  PORT5.ICR.BYTE = 0x30;

  /* We use CS3, CS4, and CS5.  */
  IOPORT.PF0CSE.BYTE = 0x38;

  /* CS5-C on P2.5, CS4-C on P2.4 */
  IOPORT.PF1CSS.BYTE = 0x0a;

  /* CS3-B on P7.3 */
  IOPORT.PF2CSS.BYTE = 0x40;

  /* We use all the address pins */
  IOPORT.PF3BUS.BYTE = 0xff;
  IOPORT.PF4BUS.BYTE = 0xff;

  /* PC7 for A16-A23, D0-D15, P5.1 is WR1  */
  IOPORT.PF5BUS.BYTE = 0x70;

  /* WAIT-B on P5.5, SDRAM enabled */
  IOPORT.PF6BUS.BYTE = 0xd1;
  /* 16-bit SDRAM, disabled - this gives us the 10 clocks we need to stabilize.  */
  BSC.SDCCR.BYTE = 0x00;

  /* IRQ 8 through 15 are -B (P4) not -A (P0)  */
  IOPORT.PF8IRQ.BYTE = 0xaf;
  /* IRQ 2 through 7 are -B (P1) not -A (P3)  */
  IOPORT.PF9IRQ.BYTE = 0xfc;

  /* Ethernet - RMII mode with minimal pins */
  IOPORT.PFENET.BYTE = 0x80;

  SYSTEM.SYSCR0.WORD = 0x5a03;	/* ROM enabled, BUS enabled */

  /* CS3: native endian, 16-bit, enabled.  */
  BSC.CS3CR.WORD = 0x0001;
  BSC.CS3WCR2.BIT.CSON = 0;

  BSC.CS3WCR2.BIT.RDON = 1;
  BSC.CS3WCR1.BIT.CSRWAIT = 5;
  BSC.CS3WCR2.BIT.CSROFF = 1;

  BSC.CS3WCR2.BIT.WDON = 0;
  BSC.CS3WCR2.BIT.WRON = 0;
  BSC.CS3WCR1.BIT.CSWWAIT = 0;
  BSC.CS3WCR2.BIT.WDOFF = 1;
  BSC.CS3WCR2.BIT.CSWOFF = 1;

  BSC.CS3REC.BIT.RRCV = 0;
  BSC.CS3REC.BIT.WRCV = 0;

  /* CS4: native endian, 16-bit, enabled, fast  */
  BSC.CS4CR.WORD = 0x0001;
  BSC.CS4WCR2.BIT.CSON = 0;
  BSC.CS4MOD.BIT.EWENB = 1;

  BSC.CS4WCR2.BIT.RDON = 1;
  BSC.CS4WCR1.BIT.CSRWAIT = 5;
  BSC.CS4WCR2.BIT.CSROFF = 1;

  BSC.CS4WCR2.BIT.WDON = 1;
  BSC.CS4WCR2.BIT.WRON = 2;
  BSC.CS4WCR1.BIT.CSWWAIT = 3;
  BSC.CS4WCR2.BIT.WDOFF = 0;
  BSC.CS4WCR2.BIT.CSWOFF = 0;

  BSC.CS4REC.BIT.RRCV = 0;
  BSC.CS4REC.BIT.WRCV = 0;

  /* CS5: native endian, 8-bit, enabled.  */
  BSC.CS5CR.WORD = 0x0021;
  BSC.CS5WCR2.BIT.CSON = 0;

  BSC.CS5WCR2.BIT.RDON = 1;
  BSC.CS5WCR1.BIT.CSRWAIT = 2;
  BSC.CS5WCR2.BIT.CSROFF = 1;

  BSC.CS5WCR2.BIT.WDON = 0;
  BSC.CS5WCR2.BIT.WRON = 1;
  BSC.CS5WCR1.BIT.CSWWAIT = 3;
  BSC.CS5WCR2.BIT.WDOFF = 1;
  BSC.CS5WCR2.BIT.CSWOFF = 1;

  BSC.CS5REC.BIT.RRCV = 0;
  BSC.CS5REC.BIT.WRCV = 0;


  /* AN3 and AN4 are analog inputs.  Set them for 14-bit continuous
     conversion.  */
  S12AD.ADANS.WORD = 0x0018;
  S12AD.ADADS.WORD = 0x0018;
  S12AD.ADADC.BYTE = 0x03;
  S12AD.ADCSR.BYTE = 0x4c;
  S12AD.ADCSR.BYTE = 0xcc; /* start conversions */

  DA.DACR.BYTE = 0xe0;
  DA.DADPR.BYTE = 0x80;

  /* Real Time Clock */

  RTC.RSECCNT.BYTE = 0;
  RTC.RMINCNT.BYTE = 0;
  RTC.RHRCNT.BYTE = 0;
  RTC.RDAYCNT.BYTE = 0;
  RTC.RMONCNT.BYTE = 9;
  RTC.RYRCNT.WORD = 0x2010;
  RTC.RCR2.BYTE = 0x03;

  /* SDRAM */

  /* At least 10 clocks must pass since we enabled the SDRAM pins above.  */
  /* Continuous read enabled */
  BSC.SDAMOD.BYTE = 1;
  /* precharge 3 cycles, two refresh 4 cycles */
  BSC.SDIR.WORD = 0x0122;
  /* Initialize */
  BSC.SDICR.BYTE = 0x01;
  while (BSC.SDSR.BYTE)
    ;
  /* SDRAM mode register */
  BSC.SDMOD.WORD = 0x0220;
  while (BSC.SDSR.BYTE)
    ;
  /* Same endian as MCU */
  BSC.SDCMOD.BYTE = 0x00;
  /* CL = 2, WR = 2, RP = 1, RCD = 1, RAS = 3 */
  BSC.SDTR.LONG = 0x00020102;
  /* 9-bit address shift */
  BSC.SDADR.BYTE = 1;
  /* Auto-refresh: 64 ms / 20 ns */
  BSC.SDRFCR.WORD = 3200 | 0xf000;
  BSC.SDRFEN.BYTE = 1;
  /* 16-bit, enabled */
  BSC.SDCCR.BYTE = 0x01;

  /* RMII mode, no extra pins */
  IOPORT.PFENET.BYTE = 0x80;
  /* MDIO */
  PORT7.ICR.BIT.B1 = 1;

  /* FPGA */
  PORT9.DR.BIT.B1 = 0;

  setup_interrupts ();
  tty_init ();
  timer_init ();
}
